body {
	max-width: 1100px;
	margin: 0 auto;
	font-size: 17px;
	font-family: Helvetica Neue, Helvetica, Arial, sans-serif;
	letter-spacing: .01rem;line-height: 130%;
}

small{
	font-size: 15;
    position: absolute;
    right: 230px;
    word-wrap: break-word;
}
